;;; -*- Mode:Lisp; Package:USER; Syntax:COMMON-LISP; Base:10 -*-

#+(or GCL LUCID) (in-package :user)
#-(or GCL LUCID) (in-package :cl-user)


;;; Copyright (c) 1999 by Micheal Hewett, hewett@cs.utexas.edu

;;;;----------------  SFS v1.0  ---------------------------------
;;;
;;; sfs.lisp        Setup, load, compile the Simple Frame System
;;;
;;; 20 Jun 1994   Mike Hewett
;;; 17 Oct 1996   (mh) Updated for Algernon v3.0 (AAM based)
;;;  6 Apr 1999   (mh) Repackaged for public consumption
;;;
;;;;-----------------------------------------------------------------  
;;; Modified by BJK (1-14-92) for the ES version.
;;; Modified by MH  (6-20-94) for Algernon v2
;;; Restructured by MH  (7 Feb 96) for Algernon v2

;;;==================================================================
;;; SETUP Instructions
;;;------------------------------------------------------------------
;;;
;;; This file defines the variables used to locate the SFS source
;;; and binary files.  Actual loading is performed by "src/sfs-system.lisp".
;;;
;;; For each configuration, load a file containing settings for the
;;; necessary variables:
;;;
;;;    *SFS-ROOT-DIR*                part of the path common to both source and object files.
;;;    *SFS-BIN-DIR*                 name of the subdirectory used for object files.
;;;    *SFS-LISP-FILE-EXTENSION*     typically ".lisp".
;;;    *SFS-OBJECT-FILE-EXTENSION*   equivalent of ".lisp" for compiled files.
;;;    *SFS-PATH-TERMINATOR*         the character between directories ('/' for UNIX).
;;;
;;; You should create a configuration file "cfg-X" for your configuration X, then
;;; add a line below to cause it to be loaded in combination with any
;;; *FEATURE* components you have in your environment.  You can set the
;;; appropriate features in the LISP init file.  
;;;
;;;-----------------------------------------------------------------------
;;; Init file information for various LISPs:
;;;
;;;    LISP         init file automatically loaded
;;;    ----         ------------------------------
;;;    GCL          ./init.lsp            <-- loads from current directory or
;;;                 $HOME/init.lsp        <-- home directory if none found in ".".
;;;
;;;    Allegro      $HOME/.clinit.cl
;;;
;;;    Lucid        $HOME/lisp-init.lisp
;;;
;;;    Macintosh    init.lisp             <-- Home directory of MCL (??)
;;;
;;;-----------------------------------------------------------------------


(push :SFS        *features*)
(push :SFS-1      *features*)
(push :SFS-1-OKBC *features*)


;;; Standard configurations for the AI group at the University of Texas.

#+(and :SUN    :ALLEGRO-V5.0)
(load "/u/qr/sfs/config/cfg-SUN-ALLEGRO-5-0.lisp")
#+(and :LINUX  :ALLEGRO-V5.0)
(load "/u/qr/sfs/config/cfg-LINUX-ALLEGRO-5-0.lisp")
#+(and         :GCL)
(load "/u/qr/sfs/config/cfg-UNIX-GCL.lisp")


(when (not (boundp '*sfs-root-dir*))
  (break "SFS is not configured for this LISP."))


;;; ---------  Set up the paths  --------------

;;; Main SFS directory in public:
(defparameter @sfs-path  *sfs-root-dir*  "Path to main SFS directory")


;;; Then the other paths:
(defparameter @sfs-source-path
  (format nil "~asrc~A" @sfs-path *SFS-PATH-TERMINATOR*) 
  "Path to source files")

(defparameter @sfs-object-path
  (format nil "~abins~A~A" @sfs-path *SFS-PATH-TERMINATOR* *SFS-BIN-DIR*)
  "Path to object files.")

(defparameter @sfs-help-path
  (format nil "~ahelp~A" @sfs-path *SFS-PATH-TERMINATOR*))

(defparameter @sfs-example-path
  (format nil "~aexamples~a" @sfs-path *SFS-PATH-TERMINATOR*))

(defparameter @sfs-gui-path
  (format nil "~agui~a" @sfs-path *SFS-PATH-TERMINATOR*))



;;; And the suffixes for various kinds of files:
(defparameter @sfs-lisp-file-extension *SFS-LISP-FILE-EXTENSION*)

(defparameter @sfs-obj-file-extension  *SFS-OBJECT-FILE-EXTENSION*)

;;; One last global:
(defvar *sfs-load-times* nil "Assoc list of files and when they were loaded.")


;;; --------------------------------------------------
;;; Some useful pathname construction routines

(defun sfs-source-file-name (filename &OPTIONAL (path @sfs-source-path))
  (concatenate 'string path filename @sfs-lisp-file-extension)
  )

(defun sfs-object-file-name (filename &OPTIONAL (path @sfs-object-path))
  (concatenate 'string path filename @sfs-obj-file-extension)
  )

(defun sfs-example-source-file-name (filename &OPTIONAL (path @sfs-example-path))
  (concatenate 'string path filename @sfs-lisp-file-extension)
  )

(defun sfs-example-object-file-name (filename &OPTIONAL (path @sfs-example-path))
  (concatenate 'string path filename @sfs-obj-file-extension)
  )



;;; Then load the rest.

;;; Load the package first.
;; Must load the package first, else the system file won't load correctly.
(if (probe-file (sfs-object-file-name "sfs-package"))
    (load (sfs-object-file-name "sfs-package"))
  ;;else
  (load (sfs-source-file-name "sfs-package")))
  
(if (probe-file (sfs-object-file-name "sfs-system"))
    (load (sfs-object-file-name "sfs-system"))
  ;;else
  (load (sfs-source-file-name "sfs-system")))

(if (probe-file (sfs-object-file-name "gfp-package"))
    (load (sfs-object-file-name "gfp-package"))
  ;;else
  (load (sfs-source-file-name "gfp-package")))
  
(if (probe-file (sfs-object-file-name "gfp-system"))
    (load (sfs-object-file-name "gfp-system"))
  ;;else
  (load (sfs-source-file-name "gfp-system")))


;;; Load the Algernon modules
(load-or-compile-sfs)
(load-or-compile-gfp)
