;;;; -*- Mode:Lisp; Package:CL-USER; Syntax:COMMON-LISP; Base:10 -*-

(in-package :CL-USER)

;;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
;;; Copyright (c) 1996 by Micheal Scott Hewett
;;;
;;; This code may be used by anyone for any project, but may not
;;; be sold in source or object form without permission.
;;; If in doubt, follow the GNU "copyleft" guidelines.
;;;
;;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
;;; (Contact: hewett@cs.utexas.edu or hewett@cs.stanford.edu)
;;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
;;;
;;;  A file that loads all the SFS files.
;;;
;;; 21 Aug 1996 (mh)  Modified to use the public name module from Algernon.
;;; 22 Aug 1996 (mh)  Added KB-GET-SLOT-DOMAINS
;;; 27 Aug 1996 (mh)  Added ALL-SLOTS-OF-FRAME
;;; 15 Oct 1996 (mh)  Shifted 'join' part of retrieval to the SFS.
;;; 18 Oct 1996 (mh)  Generalized load procedure.
;;; 12 Nov 1996 (mh)  Added unify file.
;;;
;;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 


(defparameter *ALL-SFS-FILES*
    '("monitor"
      "name-manager"
      "unify"
      "sfs-core"
      "version"))



;;; -------- Functions to load files ---------------

(defun check-sfs-file-mod-times (files)

  (let ((need-recompile   NIL))
    (dolist (file files)

      (let ((lisp-file   (sfs-source-file-name file))
	    (binary-file (sfs-object-file-name file))
	    )

	(cond ((probe-file lisp-file)
	       (cond ((probe-file binary-file)
		      (let ((lisp-date   (file-write-date lisp-file))
			    (binary-date (file-write-date binary-file)))
			(when (> lisp-date binary-date)
			  (setq need-recompile T))
			))

		     (T    (setq need-recompile T))
		     ))
	      (T
	       (format *error-output* "No such file: ~(~a~)." lisp-file))))

      (when need-recompile (return))
      )

    need-recompile
    )
  )


;; Always use LOAD-OR-COMPILE-SFS one to load the system.
(defun load-or-compile-sfs ()

  (if (check-sfs-file-mod-times *ALL-SFS-FILES*)
      (progn
	(format *trace-output* "~2%Some SFS source files have been updated.")
	(format *trace-output* "~%Compiling and loading SFS...")
	(compile-sfs))
    ;;else
    (progn
      (format *trace-output* "~2%Loading SFS...")
      (load-sfs)
      )
    )
  )


(defun compile-sfs ()

  (when (fboundp 'SFS::kb-reset)
    (sfs::kb-reset))

  ;; Load the source files first
  (dolist (filename *ALL-SFS-FILES*)
    (load-sfs-source-file filename))

  ;; Then compile all the files
  ;; Inefficient, but fast enough on current systems.
  ;; And guaranteed to be effective.
  (dolist (filename *ALL-SFS-FILES*)
    (compile-sfs-file filename))

  (load-sfs)
  )


(defun load-sfs ()

  (when (fboundp 'SFS::KB-RESET)
    (sfs::kb-reset))

  (dolist (filename  *ALL-SFS-FILES*)
    (load-sfs-file filename))
  )


;;; -----  Functions for handling individual files


(defun compile-sfs-file (filename)
  (compile-file (sfs-source-file-name filename) :output-file (sfs-object-file-name filename))
  )


(defun load-sfs-source-file (filename)


  (let ((src-filename  (sfs-source-file-name filename)))
    (if (probe-file src-filename)
	(load src-filename)
      ;;else
        NIL
	))
  )

	       
(defun load-sfs-object-file (filename)

  (let ((obj-filename  (sfs-object-file-name filename)))
    (if (probe-file obj-filename)
	(load obj-filename)
      ;;else
        NIL
	))
  )


;;; Useful general function	       
(defun load-sfs-file (filename)

  (or (load-sfs-object-file  filename)
      (load-sfs-source-file  filename)
      )
  )

